/*
 * Copyright (c) 2012 TopCoder, Inc. All rights reserved.
 * 
 */

/**
 * UI scripts for the different pages.
 * 
 * @author hanno
 * @version 1.0
 */

var tableIds = [];
var currentTable = 0;
var submitted = false;

/**
 * Set up the UI logic.
 */
$(function () {
    /*call  timepicker*/
    if ($("body").find('.timepicker').length > 0) {
        $('.timepicker').timepicker();
    }

    /*call datepicker*/
    if ($("body").find('.datepicker').length > 0) {
        $('.datepicker').datepicker();
    }

    /*expand and hide the content*/
    $('.hideExpand').click(function () {
        if ($(this).hasClass('hideBtn')) {
            $(this).removeClass('hideBtn').addClass('expandBtn');
            $('.resultTable').hide();
            $(this).parents('.dayRow').find('.dayContent').hide();
        } else if ($(this).hasClass('expandBtn')) {
            $(this).removeClass('expandBtn').addClass('hideBtn');
            showCurrentTable();
            $(this).parents('.dayRow').find('.dayContent').show();
        }
    });

    /*setpoint single day or 7 days a week show hide function*/
    $('.selectRow input').click(function () {
        switch ($(this).attr('id')) {
        case "sd":
            $('.forSD').show();
            $('.fordw').hide();
            break;
        case "dw":
            $('.forSD').hide();
            $('.fordw').show();
            break;
        default:
            break;
        }
    });

    $('.datepicker,select,#inputLoc,#inputFile').change(function () {
        validateInput();
    });

    $('.datepicker,select,#inputLoc').keyup(function () {
        validateInput();
    });

    tableIds.sort();
    $('#totalPages').text(tableIds.length);
    showCurrentTable();

    /*preselect dropdowns if available*/
    $('#degreeDayTypeDropDown option[value="' + $('#hiddenDegreeDayType').text() + '"]').attr('selected', 'selected');
    $('#degreeTypeDropDown option[value="' + $('#hiddenDegreeType').text() + '"]').attr('selected', 'selected');

    if ($('#hiddenSetpointSchedule').text() == 'SEVEN_DAYS') {
        $('#dw').click();
    } else {
        $('#sd').click();
    }
});

/**
 * Shows the current table for paging.
 */
function showCurrentTable () {
    $('.resultTable').hide();
    $('#tbl_' + tableIds[currentTable]).show();
    $('#currentPage').text(currentTable + 1);
    $('#headerDate').text($('#span_' + tableIds[currentTable]).text());

    if (currentTable == tableIds.length - 1) {
        $('#btnNext').hide();
        $('#spanNext').show();
        $('#btnLast').hide();
        $('#spanLast').show();
    } else {
        $('#btnNext').show();
        $('#spanNext').hide();
        $('#btnLast').show();
        $('#spanLast').hide();
    }

    if (currentTable == 0) {
        $('#btnPrev').hide();
        $('#spanPrev').show();
        $('#btnFirst').hide();
        $('#spanFirst').show();
    } else {
        $('#btnPrev').show();
        $('#spanPrev').hide();
        $('#btnFirst').show();
        $('#spanFirst').hide();
    }
}

/**
 * Switch to the next table.
 */
function showNextTable () {
    if (currentTable < tableIds.length - 1) {
        currentTable++;
        showCurrentTable();
    }
}

/**
 * Switch to the last table.
 */
function showLastTable () {
    currentTable = tableIds.length - 1;
    showCurrentTable();
}

/**
 * Switch to the previous table.
 */
function showPreviousTable () {
    if (currentTable != 0) {
        currentTable--;
        showCurrentTable();
    }
}

/**
 * Switch to the first table.
 */
function showFirstTable () {
    currentTable = 0;
    showCurrentTable();
}

/**
 * Export weather date.
 * 
 * @param direction
 *            the direction (VERTICAL or HORIZONTAL).
 */
function exportData (direction) {
    $('#exportDir').val(direction);
    $('#exportForm').submit();
}

/**
 * Search degree day data.
 */
function submitSearch (action) {
    $('#searchForm').attr('action', action);
    submitted = true;
    if (validateInput(false)) {
        $('#searchForm').submit();
    }
}

/**
 * Export schedule.
 */
function exportSetpointSchedule () {
    $('#searchForm').attr('action', 'exportSetpointSchedule.do');
    $('#searchForm').submit();
}

/**
 * Import schedule.
 */
function importSetpointSchedule () {
    $('#searchForm').attr('action', 'importSetpointSchedule.do');
    submitted = true;
    if (validateInput(true)) {
        $('#searchForm').submit();
    }
}

/**
 * Validates the input fields.
 * 
 * @returns true if all fields are ok
 */
function validateInput (forImport) {
    calculateTimeSpan();

    if (!submitted) {
        return;
    }

    $('*').removeClass('error');
    $('.errorMessage').css('visibility', 'hidden');

    var valid = true;

    if (!forImport) {
        $('.datepicker').each(function () {
            if (!splitDate($(this).val())) {
                valid = false;
                $(this).addClass('error');
            }
        });

        $('select').each(function () {
            if ($(this).val() == 'Please select') {
                valid = false;
                $(this).addClass('error');
            }
        });

        if (!$('#inputLoc').val()) {
            valid = false;
            $('#inputLoc').addClass('error');
        }
    } else {
        if (!$('#inputFile').val()) {
            valid = false;
            $('#inputFile').addClass('error');
        }
    }

    if (!valid) {
        $('.errorMessage').css('visibility', 'visible');
    }

    return valid;
}

/**
 * Calculate the time span.
 */
function calculateTimeSpan () {
    var begin = splitDate($('#beginningDate').val());
    var end = splitDate($('#endDate').val());
    var txt = '';

    if (begin && end) {
        var dif = new Date(end[2], end[0], end[1], 0, 0, 0, 0).getTime()
                - new Date(begin[2], begin[0], begin[1], 0, 0, 0, 0).getTime();
        var difDays = Math.floor((1 + dif / (24 * 60 * 60 * 1000)));
        if (difDays > 0) {
            txt = '(' + difDays + ' day' + (difDays > 1 ? 's' : '') + ' to query)';
        }
    }

    $('#timeSpan').html(txt);
}

/**
 * Split date in year, month, day.
 * 
 * @param value
 *            date value
 * @returns splitted date or null
 */
function splitDate (value) {
    if (value != 'mm/dd/yyyy') {
        var splitted = value.split('/');
        if (splitted.length == 3 && splitted[0] > 0 && splitted[0] <= 12 && splitted[1] > 0 && splitted[1] <= 31
                && splitted[2] > 1000) {
            return splitted;
        }
    }
    return null;
}

/**
 * Copy 7 setpoints from 'from' to 'to'.
 * 
 * @param from
 *            from index
 * @param to
 *            to index
 */
function copy (from, to) {
    for ( var i = 0; i < 7; i++) {
        $('#times_' + (to + i)).val($('#times_' + (from + i)).val());
        $('#setpoints_' + (to + i)).val($('#setpoints_' + (from + i)).val());
    }
}

/**
 * Clear search fields.
 */
function clearDDSearch () {
    $('#degreeDayTypeDropDown option[value="Please select"]').attr('selected', 'selected');
    $('#degreeTypeDropDown option[value="FAHRENHEIT"]').attr('selected', 'selected');
    $('#inputLoc').val('');
    $('#inputFile').val('');
    $('#beginningDate').val('mm/dd/yyyy');
    $('#endDate').val('mm/dd/yyyy');
}

/**
 * Clear search fields.
 */
function clearWDSearch () {
    $('#inputLoc').val('');
    $('#beginningDate').val('mm/dd/yyyy');
    $('#endDate').val('mm/dd/yyyy');
}

/**
 * Clear search fields.
 */
function clearSP () {
    $('.setPoint input[type="text"]').val('');
    $('.timepicker').val('HH:MM');
}