/*
 * Copyright (C) 2012 TopCoder Inc., All Rights Reserved.
 */
package com.enerallies.weather.entity;

import java.util.List;

/**
 * Entity for degree data for a station.
 *
 * <p>
 * <b>Thread Safety:</b> This class is not thread safe since it is mutable.
 * </p>
 *
 * @author Standlove, hanno
 * @version 1.0
 */
public class DegreeDayData {
    /**
     * The weather station.
     */
    private String station;

    /**
     * The id of the station.
     */
    private String stationId;

    /**
     * The type in which the degree day was calculated.
     */
    private DegreeType degreeType;

    /**
     * Degree day type.
     */
    private DegreeDayType degreeDayType;

    /**
     * List of values.
     */
    private List<DegreeDay> degreeDays;

    /**
     * The original search criteria.
     */
    private WeatherDataSearchCriteria criteria;

    /**
     * The total DD.
     */
    private double totalDD;

    /**
     * Empty constructor for entity class.
     */
    public DegreeDayData() {
        // empty
    }

    /**
     * Returns the station.
     *
     * @return the station
     */
    public String getStation() {
        return station;
    }

    /**
     * Sets the station.
     *
     * @param station
     *            the new station to set
     */
    public void setStation(String station) {
        this.station = station;
    }

    /**
     * Returns the stationId.
     *
     * @return the stationId
     */
    public String getStationId() {
        return stationId;
    }

    /**
     * Sets the stationId.
     *
     * @param stationId
     *            the new stationId to set
     */
    public void setStationId(String stationId) {
        this.stationId = stationId;
    }

    /**
     * Returns the degreeDays.
     *
     * @return the degreeDays
     */
    public List<DegreeDay> getDegreeDays() {
        return degreeDays;
    }

    /**
     * Sets the degreeDays.
     *
     * @param degreeDays
     *            the new degreeDays to set
     */
    public void setDegreeDays(List<DegreeDay> degreeDays) {
        this.degreeDays = degreeDays;
    }

    /**
     * Returns the degreeType.
     *
     * @return the degreeType
     */
    public DegreeType getDegreeType() {
        return degreeType;
    }

    /**
     * Sets the degreeType.
     *
     * @param degreeType
     *            the new degreeType to set
     */
    public void setDegreeType(DegreeType degreeType) {
        this.degreeType = degreeType;
    }

    /**
     * Returns the degreeDayType.
     *
     * @return the degreeDayType
     */
    public DegreeDayType getDegreeDayType() {
        return degreeDayType;
    }

    /**
     * Sets the degreeDayType.
     *
     * @param degreeDayType
     *            the new degreeDayType to set
     */
    public void setDegreeDayType(DegreeDayType degreeDayType) {
        this.degreeDayType = degreeDayType;
    }

    /**
     * Returns the criteria.
     *
     * @return the criteria
     */
    public WeatherDataSearchCriteria getCriteria() {
        return criteria;
    }

    /**
     * Sets the criteria.
     *
     * @param criteria
     *            the new criteria to set
     */
    public void setCriteria(WeatherDataSearchCriteria criteria) {
        this.criteria = criteria;
    }

    /**
     * Returns the totalDD.
     *
     * @return the totalDD
     */
    public double getTotalDD() {
        return totalDD;
    }

    /**
     * Sets the totalDD.
     *
     * @param totalDD
     *            the new totalDD to set
     */
    public void setTotalDD(double totalDD) {
        this.totalDD = totalDD;
    }
}
