/*
 * Copyright (C) 2012 TopCoder Inc., All Rights Reserved.
 */
package com.enerallies.weather.entity;

import java.util.Date;

/**
 * Entity for detailed weather data.
 *
 * <p>
 * <b>Thread Safety:</b> This class is not thread safe since it is mutable.
 * </p>
 *
 * @author Standlove, hanno
 * @version 1.0
 */
public class WeatherData {
    /**
     * The time.
     */
    private Date time;

    /**
     * The temperature.
     */
    private Double temperature;

    /**
     * The dew point.
     */
    private Double dewPoint;

    /**
     * The humidity.
     */
    private Integer humidity;

    /**
     * The visibility.
     */
    private Double visibility;

    /**
     * The wind direction.
     */
    private String windDirection;

    /**
     * The wind speed.
     */
    private Double windSpeed;

    /**
     * The gust speed.
     */
    private Double gustSpeed;

    /**
     * The precip.
     */
    private Double precip;

    /**
     * The date in UTC.
     */
    private Date utcDate;

    /**
     * Timezone where data was recorded.
     */
    private String timeZone;

    /**
     * Search criteria.
     */
    private WeatherDataSearchCriteria criteria;

    /**
     * Empty constructor for entity class.
     */
    public WeatherData() {
        // empty
    }

    /**
     * Returns the time.
     *
     * @return the time
     */
    public Date getTime() {
        return time;
    }

    /**
     * Sets the time.
     *
     * @param time
     *            the new time to set
     */
    public void setTime(Date time) {
        this.time = time;
    }

    /**
     * Returns the temperature.
     *
     * @return the temperature
     */
    public Double getTemperature() {
        return temperature;
    }

    /**
     * Sets the temperature.
     *
     * @param temperature
     *            the new temperature to set
     */
    public void setTemperature(Double temperature) {
        this.temperature = temperature;
    }

    /**
     * Returns the dewPoint.
     *
     * @return the dewPoint
     */
    public Double getDewPoint() {
        return dewPoint;
    }

    /**
     * Sets the dewPoint.
     *
     * @param dewPoint
     *            the new dewPoint to set
     */
    public void setDewPoint(Double dewPoint) {
        this.dewPoint = dewPoint;
    }

    /**
     * Returns the humidity.
     *
     * @return the humidity
     */
    public Integer getHumidity() {
        return humidity;
    }

    /**
     * Sets the humidity.
     *
     * @param humidity
     *            the new humidity to set
     */
    public void setHumidity(Integer humidity) {
        this.humidity = humidity;
    }

    /**
     * Returns the visibility.
     *
     * @return the visibility
     */
    public Double getVisibility() {
        return visibility;
    }

    /**
     * Sets the visibility.
     *
     * @param visibility
     *            the new visibility to set
     */
    public void setVisibility(Double visibility) {
        this.visibility = visibility;
    }

    /**
     * Returns the windDirection.
     *
     * @return the windDirection
     */
    public String getWindDirection() {
        return windDirection;
    }

    /**
     * Sets the windDirection.
     *
     * @param windDirection
     *            the new windDirection to set
     */
    public void setWindDirection(String windDirection) {
        this.windDirection = windDirection;
    }

    /**
     * Returns the windSpeed.
     *
     * @return the windSpeed
     */
    public Double getWindSpeed() {
        return windSpeed;
    }

    /**
     * Sets the windSpeed.
     *
     * @param windSpeed
     *            the new windSpeed to set
     */
    public void setWindSpeed(Double windSpeed) {
        this.windSpeed = windSpeed;
    }

    /**
     * Returns the gustSpeed.
     *
     * @return the gustSpeed
     */
    public Double getGustSpeed() {
        return gustSpeed;
    }

    /**
     * Sets the gustSpeed.
     *
     * @param gustSpeed
     *            the new gustSpeed to set
     */
    public void setGustSpeed(Double gustSpeed) {
        this.gustSpeed = gustSpeed;
    }

    /**
     * Returns the precip.
     *
     * @return the precip
     */
    public Double getPrecip() {
        return precip;
    }

    /**
     * Sets the precip.
     *
     * @param precip
     *            the new precip to set
     */
    public void setPrecip(Double precip) {
        this.precip = precip;
    }

    /**
     * Returns the utcDate.
     *
     * @return the utcDate
     */
    public Date getUtcDate() {
        return utcDate;
    }

    /**
     * Sets the utcDate.
     *
     * @param utcDate
     *            the new utcDate to set
     */
    public void setUtcDate(Date utcDate) {
        this.utcDate = utcDate;
    }

    /**
     * Returns the timeZone.
     *
     * @return the timeZone
     */
    public String getTimeZone() {
        return timeZone;
    }

    /**
     * Sets the timeZone.
     *
     * @param timeZone
     *            the new timeZone to set
     */
    public void setTimeZone(String timeZone) {
        this.timeZone = timeZone;
    }

    /**
     * Returns the criteria.
     *
     * @return the criteria
     */
    public WeatherDataSearchCriteria getCriteria() {
        return criteria;
    }

    /**
     * Sets the criteria.
     *
     * @param criteria
     *            the new criteria to set
     */
    public void setCriteria(WeatherDataSearchCriteria criteria) {
        this.criteria = criteria;
    }
}
