/*
 * Copyright (C) 2012 TopCoder Inc., All Rights Reserved.
 */
package com.enerallies.weather.service;

import java.util.Calendar;
import java.util.List;
import java.util.TimeZone;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;

import com.enerallies.weather.entity.WeatherData;

/**
 * The response from the wunderground API annotated for JAXB.
 *
 * @author hanno
 * @version 1.0
 */
@XmlRootElement
class Response {
    /**
     * The error element.
     */
    private Error error;

    /**
     * The history element.
     */
    private History history;

    /**
     * Formatted date used to get this response.
     */
    private String formattedDate;

    /**
     * Empty constructor.
     */
    public Response() {
        // empty
    }

    /**
     * Returns the error.
     *
     * @return the error
     */
    @XmlElement
    public Error getError() {
        return error;
    }

    /**
     * Sets the error.
     *
     * @param error
     *            the new error to set
     */
    public void setError(Error error) {
        this.error = error;
    }

    /**
     * Returns the history.
     *
     * @return the history
     */
    @XmlElement
    public History getHistory() {
        return history;
    }

    /**
     * Sets the history.
     *
     * @param history
     *            the new history to set
     */
    public void setHistory(History history) {
        this.history = history;
    }

    /**
     * History response element.
     *
     * @author hanno
     * @version 1.0
     */
    public static class History {
        /**
         * The date element.
         */
        private ResponseDate date;

        /**
         * The list of observations.
         */
        private List<Observation> observations;

        /**
         * Empty constructor.
         */
        public History() {
            // empty
        }

        /**
         * Returns the date.
         *
         * @return the date
         */
        @XmlElement
        public ResponseDate getDate() {
            return date;
        }

        /**
         * Sets the date.
         *
         * @param date
         *            the new date to set
         */
        public void setDate(ResponseDate date) {
            this.date = date;
        }

        /**
         * Returns the observations.
         *
         * @return the observations
         */
        @XmlElementWrapper(name = "observations")
        @XmlElement(name = "observation")
        public List<Observation> getObservations() {
            return observations;
        }

        /**
         * Sets the observations.
         *
         * @param observations
         *            the new observations to set
         */
        public void setObservations(List<Observation> observations) {
            this.observations = observations;
        }
    }

    /**
     * Observation response element.
     *
     * @author hanno
     * @version 1.0
     */
    public static class Observation implements Cloneable {
        /**
         * Unset double values.
         */
        private static final int[] UNSET_VALUES = {-999, -9999 };

        /**
         * Date element.
         */
        private ResponseDate date;

        /**
         * UtcDate element.
         */
        private ResponseDate utcDate;

        /**
         * Tempi element.
         */
        private Double temperature;

        /**
         * Dewpti element.
         */
        private Double dewPoint;

        /**
         * Hum element.
         */
        private Integer humidity;

        /**
         * Vism element.
         */
        private Double visibility;

        /**
         * Wdire element.
         */
        private String windDirection;

        /**
         * Wspdm element.
         */
        private Double windSpeed;

        /**
         * Wgustm element.
         */
        private Double gustSpeed;

        /**
         * Precipm element.
         */
        private Double precip;

        /**
         * Empty constructor.
         */
        public Observation() {
            // empty
        }

        /**
         * Returns the date.
         *
         * @return the date
         */
        @XmlElement(required = true)
        public ResponseDate getDate() {
            return date;
        }

        /**
         * Sets the date.
         *
         * @param date
         *            the new date to set
         */
        public void setDate(ResponseDate date) {
            this.date = date;
        }

        /**
         * Returns the utcDate.
         *
         * @return the utcDate
         */
        @XmlElement(required = true, name = "utcdate")
        public ResponseDate getUtcDate() {
            return utcDate;
        }

        /**
         * Sets the utcDate.
         *
         * @param utcDate
         *            the new utcDate to set
         */
        public void setUtcDate(ResponseDate utcDate) {
            this.utcDate = utcDate;
        }

        /**
         * Returns the temperature.
         *
         * @return the temperature
         */
        @XmlElement(name = "tempi")
        public Double getTemperature() {
            return temperature;
        }

        /**
         * Sets the temperature.
         *
         * @param temperature
         *            the new temperature to set
         */
        public void setTemperature(Double temperature) {
            this.temperature = checked(temperature);
        }

        /**
         * Returns the dewPoint.
         *
         * @return the dewPoint
         */
        @XmlElement(name = "dewpti")
        public Double getDewPoint() {
            return dewPoint;
        }

        /**
         * Sets the dewPoint.
         *
         * @param dewPoint
         *            the new dewPoint to set
         */
        public void setDewPoint(Double dewPoint) {
            this.dewPoint = checked(dewPoint);
        }

        /**
         * Returns the humidity.
         *
         * @return the humidity
         */
        @XmlElement(name = "hum")
        public Integer getHumidity() {
            return humidity;
        }

        /**
         * Sets the humidity.
         *
         * @param humidity
         *            the new humidity to set
         */
        public void setHumidity(Integer humidity) {
            this.humidity = checked(humidity);
        }

        /**
         * Returns the visibility.
         *
         * @return the visibility
         */
        @XmlElement(name = "vism")
        public Double getVisibility() {
            return visibility;
        }

        /**
         * Sets the visibility.
         *
         * @param visibility
         *            the new visibility to set
         */
        public void setVisibility(Double visibility) {
            this.visibility = checked(visibility);
        }

        /**
         * Returns the windDirection.
         *
         * @return the windDirection
         */
        @XmlElement(name = "wdire")
        public String getWindDirection() {
            return windDirection;
        }

        /**
         * Sets the windDirection.
         *
         * @param windDirection
         *            the new windDirection to set
         */
        public void setWindDirection(String windDirection) {
            this.windDirection = windDirection;
        }

        /**
         * Returns the windSpeed.
         *
         * @return the windSpeed
         */
        @XmlElement(name = "wspdm")
        public Double getWindSpeed() {
            return windSpeed;
        }

        /**
         * Sets the windSpeed.
         *
         * @param windSpeed
         *            the new windSpeed to set
         */
        public void setWindSpeed(Double windSpeed) {
            this.windSpeed = checked(windSpeed);
        }

        /**
         * Returns the gustSpeed.
         *
         * @return the gustSpeed
         */
        @XmlElement(name = "wgustm")
        public Double getGustSpeed() {
            return gustSpeed;
        }

        /**
         * Sets the gustSpeed.
         *
         * @param gustSpeed
         *            the new gustSpeed to set
         */
        public void setGustSpeed(Double gustSpeed) {
            this.gustSpeed = checked(gustSpeed);
        }

        /**
         * Returns the precip.
         *
         * @return the precip
         */
        @XmlElement(name = "precipm")
        public Double getPrecip() {
            return precip;
        }

        /**
         * Sets the precip.
         *
         * @param precip
         *            the new precip to set
         */
        public void setPrecip(Double precip) {
            this.precip = checked(precip);
        }

        /**
         * Returns a weather data.
         *
         * @return a weather data.
         * @throws WeatherDataServiceException
         *             if an error occurs
         */
        public WeatherData toWeatherData() throws WeatherDataServiceException {
            WeatherData wd = new WeatherData();

            wd.setTime(getDate().toDate().getTime());
            wd.setTimeZone(getDate().getTimeZoneName());
            wd.setTemperature(checked(getTemperature()));
            wd.setDewPoint(checked(getDewPoint()));
            wd.setHumidity(checked(getHumidity()));
            wd.setVisibility(checked(getVisibility()));
            wd.setWindDirection(getWindDirection());
            wd.setWindSpeed(checked(getWindSpeed()));
            wd.setGustSpeed(checked(getGustSpeed()));
            wd.setPrecip(checked(getPrecip()));
            wd.setUtcDate(getUtcDate().toDate().getTime());

            return wd;
        }

        /**
         * Checks the double. Unset values will get null.
         *
         * @param d
         *            double value
         * @return checked double or null
         * @param <T>
         *            type of number
         */
        private <T extends Number> T checked(T d) {
            if (d == null) {
                return null;
            }
            for (int i : UNSET_VALUES) {
                if (i == d.intValue()) {
                    return null;
                }
            }
            return d;
        }

        /**
         * Clone.
         *
         * @return a clone.
         */
        @Override
        public Observation clone() {
            try {
                Observation clone = (Observation) super.clone();
                clone.date = date.clone();
                clone.utcDate = utcDate.clone();
                return clone;
            } catch (CloneNotSupportedException e) {
                // will not happen, Cloneable is implemented
            }
            return null;
        }
    }

    /**
     * Date response element.
     *
     * @author hanno
     * @version 1.0
     */
    public static class ResponseDate implements Cloneable {
        /**
         * Year element.
         */
        private int year;

        /**
         * Mon element.
         */
        private int month;

        /**
         * Mday element.
         */
        private int dayOfMonth;

        /**
         * Hour element.
         */
        private int hour;

        /**
         * Min element.
         */
        private int minute;

        /**
         * Tzname element.
         */
        private String timeZoneName;

        /**
         * Empty constructor.
         */
        public ResponseDate() {
            // empty
        }

        /**
         * Returns the year.
         *
         * @return the year
         */
        @XmlElement(required = true)
        public int getYear() {
            return year;
        }

        /**
         * Sets the year.
         *
         * @param year
         *            the new year to set
         */
        public void setYear(int year) {
            this.year = year;
        }

        /**
         * Returns the month.
         *
         * @return the month
         */
        @XmlElement(name = "mon", required = true)
        public int getMonth() {
            return month;
        }

        /**
         * Sets the month.
         *
         * @param month
         *            the new month to set
         */
        public void setMonth(int month) {
            this.month = month;
        }

        /**
         * Returns the dayOfMonth.
         *
         * @return the dayOfMonth
         */
        @XmlElement(name = "mday", required = true)
        public int getDayOfMonth() {
            return dayOfMonth;
        }

        /**
         * Sets the dayOfMonth.
         *
         * @param dayOfMonth
         *            the new dayOfMonth to set
         */
        public void setDayOfMonth(int dayOfMonth) {
            this.dayOfMonth = dayOfMonth;
        }

        /**
         * Returns the hour.
         *
         * @return the hour
         */
        @XmlElement(required = true)
        public int getHour() {
            return hour;
        }

        /**
         * Sets the hour.
         *
         * @param hour
         *            the new hour to set
         */
        public void setHour(int hour) {
            this.hour = hour;
        }

        /**
         * Returns the minute.
         *
         * @return the minute
         */
        @XmlElement(name = "min", required = true)
        public int getMinute() {
            return minute;
        }

        /**
         * Sets the minute.
         *
         * @param minute
         *            the new minute to set
         */
        public void setMinute(int minute) {
            this.minute = minute;
        }

        /**
         * Returns the timeZoneName.
         *
         * @return the timeZoneName
         */
        @XmlElement(name = "tzname", required = true)
        public String getTimeZoneName() {
            return timeZoneName;
        }

        /**
         * Sets the timeZoneName.
         *
         * @param timeZoneName
         *            the new timeZoneName to set
         */
        public void setTimeZoneName(String timeZoneName) {
            this.timeZoneName = timeZoneName;
        }

        /**
         * Transform to java date.
         *
         * @return java date.
         * @throws WeatherDataServiceException
         *             if an error occurs
         */
        public Calendar toDate() throws WeatherDataServiceException {
            Calendar c = Calendar.getInstance(TimeZone.getTimeZone(timeZoneName));

            c.set(Calendar.YEAR, year);
            c.set(Calendar.MONTH, month - 1);
            c.set(Calendar.DATE, dayOfMonth);
            c.set(Calendar.HOUR_OF_DAY, hour);
            c.set(Calendar.MINUTE, minute);
            c.set(Calendar.SECOND, 0);
            c.set(Calendar.MILLISECOND, 0);

            return c;
        }

        /**
         * Fill values from calendar object.
         *
         * @param c
         *            calendar object
         */
        public void setDate(Calendar c) {
            this.timeZoneName = c.getTimeZone().getID();
            this.year = c.get(Calendar.YEAR);
            this.month = c.get(Calendar.MONTH) + 1;
            this.dayOfMonth = c.get(Calendar.DATE);
            this.hour = c.get(Calendar.HOUR_OF_DAY);
            this.minute = c.get(Calendar.MINUTE);
        }

        /**
         * Clone.
         *
         * @return a clone.
         */
        @Override
        public ResponseDate clone() {
            try {
                return (ResponseDate) super.clone();
            } catch (CloneNotSupportedException e) {
                // will not happen, Cloneable is implemented
            }
            return null;
        }
    }

    /**
     * Error response element.
     *
     * @author hanno
     * @version 1.0
     */
    public static class Error {
        /**
         * Error type.
         */
        private String type;

        /**
         * Error description.
         */
        private String description;

        /**
         * Empty constructor.
         */
        public Error() {
            // empty
        }

        /**
         * Returns the type.
         *
         * @return the type
         */
        public String getType() {
            return type;
        }

        /**
         * Sets the type.
         *
         * @param type
         *            the new type to set
         */
        public void setType(String type) {
            this.type = type;
        }

        /**
         * Returns the description.
         *
         * @return the description
         */
        public String getDescription() {
            return description;
        }

        /**
         * Sets the description.
         *
         * @param description
         *            the new description to set
         */
        public void setDescription(String description) {
            this.description = description;
        }
    }

    /**
     * Returns the formattedDate.
     *
     * @return the formattedDate
     */
    public String getFormattedDate() {
        return formattedDate;
    }

    /**
     * Sets the formattedDate.
     *
     * @param formattedDate
     *            the new formattedDate to set
     */
    public void setFormattedDate(String formattedDate) {
        this.formattedDate = formattedDate;
    }
}
